@@ -263,6 +263,7 @@ enum fio_ioengine_flags {
 	FIO_NODISKUTIL  = 1 << 4,       /* diskutil can't handle filename */
 	FIO_UNIDIR	= 1 << 5,	/* engine is uni-directional */
 	FIO_NOIO	= 1 << 6,	/* thread does only pseudo IO */
+	FIO_SIGQUIT	= 1 << 7,	/* needs SIGQUIT to exit */
 };
 
 enum fio_file_flags {
@@ -667,6 +668,7 @@ extern unsigned long long mlock_size;
 extern unsigned long page_mask, page_size;
 extern int read_only;
 extern int eta_print;
+extern char *job_section;
 
 extern struct thread_data *threads;
 
@@ -681,7 +683,7 @@ static inline void fio_ro_check(struct thread_data *td, struct io_u *io_u)
 }
 
 #define BLOCKS_PER_MAP		(8 * sizeof(long))
-#define TO_MAP_BLOCK(td, f, b)	((b) - ((f)->file_offset / (unsigned long long) (td)->o.rw_min_bs))
+#define TO_MAP_BLOCK(td, f, b)	(b)
 #define RAND_MAP_IDX(td, f, b)	(TO_MAP_BLOCK(td, f, b) / BLOCKS_PER_MAP)
 #define RAND_MAP_BIT(td, f, b)	(TO_MAP_BLOCK(td, f, b) & (BLOCKS_PER_MAP - 1))
 
@@ -979,6 +981,7 @@ enum {
 	FD_MEM,
 	FD_BLKTRACE,
 	FD_VERIFY,
+	FD_RANDOM,
 	FD_DEBUG_MAX,
 };
 
