@@ -17,7 +17,7 @@
 #include "fio.h"
 #include "parse.h"
 
-static char fio_version_string[] = "fio 1.18.1";
+static char fio_version_string[] = "fio 1.19";
 
 #define FIO_RANDSEED		(0xb1899bedUL)
 
@@ -34,6 +34,7 @@ int eta_print;
 unsigned long long mlock_size = 0;
 FILE *f_out = NULL;
 FILE *f_err = NULL;
+char *job_section = NULL;
 
 int write_bw_log = 0;
 int read_only = 0;
@@ -110,6 +111,11 @@ static struct option long_options[FIO_NR_OPTIONS] = {
 		.has_arg	= required_argument,
 		.val		= 'd',
 	},
+	{
+		.name		= "section",
+		.has_arg	= required_argument,
+		.val		= 'x',
+	},
 	{
 		.name		= NULL,
 	},
@@ -548,6 +554,16 @@ err:
 	return -1;
 }
 
+static int skip_this_section(const char *name)
+{
+	if (!job_section)
+		return 0;
+	if (!strncmp(name, "global", 6))
+		return 0;
+
+	return strncmp(job_section, name, strlen(job_section));
+}
+
 static int is_empty_or_comment(char *line)
 {
 	unsigned int i;
@@ -577,6 +593,7 @@ static int parse_jobs_ini(char *file, int stonewall_flag)
 	int ret = 0, stonewall;
 	int first_sect = 1;
 	int skip_fgets = 0;
+	int inside_skip = 0;
 
 	if (!strcmp(file, "-"))
 		f = stdin;
@@ -615,10 +632,18 @@ static int parse_jobs_ini(char *file, int stonewall_flag)
 		if (is_empty_or_comment(p))
 			continue;
 		if (sscanf(p, "[%255s]", name) != 1) {
+			if (inside_skip)
+				continue;
 			log_err("fio: option <%s> outside of [] job section\n", p);
 			break;
 		}
 
+		if (skip_this_section(name)) {
+			inside_skip = 1;
+			continue;
+		} else
+			inside_skip = 0;
+
 		global = !strncmp(name, "global", 6);
 
 		name[strlen(name) - 1] = '\0';
@@ -774,6 +799,8 @@ static void usage(const char *name)
 	printf("\t--showcmd\tTurn a job file into command line options\n");
 	printf("\t--eta=when\tWhen ETA estimate should be printed\n");
 	printf("\t          \tMay be \"always\", \"never\" or \"auto\"\n");
+	printf("\t--readonly\tTurn on safety read-only checks, preventing writes\n");
+	printf("\t--section=name\tOnly run specified section in job file\n");
 }
 
 #ifdef FIO_INC_DEBUG
@@ -784,10 +811,11 @@ struct debug_level debug_levels[] = {
 	{ .name = "mem",	.shift = FD_MEM, },
 	{ .name = "blktrace",	.shift = FD_BLKTRACE },
 	{ .name = "verify",	.shift = FD_VERIFY },
+	{ .name = "random",	.shift = FD_RANDOM },
 	{ },
 };
 
-static void set_debug(const char *string)
+static int set_debug(const char *string)
 {
 	struct debug_level *dl;
 	char *p = (char *) string;
@@ -803,10 +831,10 @@ static void set_debug(const char *string)
 			log_info("%s,", dl->name);
 		}
 		log_info("all\n");
-		return;
+		return 1;
 	} else if (!strcmp(string, "all")) {
 		fio_debug = ~0UL;
-		return;
+		return 0;
 	}
 
 	while ((opt = strsep(&p, ",")) != NULL) {
@@ -825,18 +853,20 @@ static void set_debug(const char *string)
 		if (!found)
 			log_err("fio: debug mask %s not found\n", opt);
 	}
+	return 0;
 }
 #else
 static void set_debug(const char *string)
 {
 	log_err("fio: debug tracing not included in build\n");
+	return 1;
 }
 #endif
 
 static int parse_cmd_line(int argc, char *argv[])
 {
 	struct thread_data *td = NULL;
-	int c, ini_idx = 0, lidx, ret, dont_add_job = 0;
+	int c, ini_idx = 0, lidx, ret = 0, do_exit = 0, exit_val = 0;
 
 	while ((c = getopt_long_only(argc, argv, "", long_options, &lidx)) != -1) {
 		switch (c) {
@@ -881,7 +911,19 @@ static int parse_cmd_line(int argc, char *argv[])
 				eta_print = FIO_ETA_NEVER;
 			break;
 		case 'd':
-			set_debug(optarg);
+			if (set_debug(optarg))
+				do_exit++;
+			break;
+		case 'x':
+			if (!strcmp(optarg, "global")) {
+				log_err("fio: can't use global as only section\n");
+				do_exit++;
+				exit_val = 1;
+				break;
+			}
+			if (job_section)
+				free(job_section);
+			job_section = strdup(optarg);
 			break;
 		case FIO_GETOPT_JOB: {
 			const char *opt = long_options[lidx].name;
@@ -896,35 +938,38 @@ static int parse_cmd_line(int argc, char *argv[])
 				td = NULL;
 			}
 			if (!td) {
+				int is_section = !strncmp(opt, "name", 4);
 				int global = 0;
 
-				if (strncmp(opt, "name", 4) ||
-				    !strncmp(val, "global", 6))
+				if (!is_section || !strncmp(val, "global", 6))
 					global = 1;
 
+				if (is_section && skip_this_section(val))
+					continue;
+
 				td = get_new_job(global, &def_thread);
 				if (!td)
 					return 0;
 			}
 
 			ret = fio_cmd_option_parse(td, opt, val);
-			if (ret)
-				dont_add_job = 1;
 			break;
 		}
 		default:
+			do_exit++;
+			exit_val = 1;
 			break;
 		}
 	}
 
+	if (do_exit)
+		exit(exit_val);
+
 	if (td) {
-		if (dont_add_job)
-			put_job(td);
-		else {
+		if (!ret)
 			ret = add_job(td, td->o.name ?: "fio", 0);
-			if (ret)
-				put_job(td);
-		}
+		if (ret)
+			put_job(td);
 	}
 
 	while (optind < argc) {
