@@ -106,7 +106,7 @@ static unsigned long get_mult_bytes(char c)
 /*
  * convert string into decimal value, noting any size suffix
  */
-static int str_to_decimal(const char *str, long long *val, int kilo)
+int str_to_decimal(const char *str, long long *val, int kilo)
 {
 	int len;
 
@@ -350,7 +350,7 @@ static int __handle_option(struct fio_option *o, const char *ptr, void *data,
 				val_store(ilp, ul1, o->off1, data);
 				val_store(ilp, ul2, o->off2, data);
 			}
-			if (!more && o->off3 && o->off4) {
+			if (o->off3 && o->off4) {
 				val_store(ilp, ul1, o->off3, data);
 				val_store(ilp, ul2, o->off4, data);
 			}
